@@ -1,8 +1,8 @@
 ;;;============================================================================
 
-;;; File: "_parms.scm", Time-stamp: <2008-02-14 13:54:34 feeley>
+;;; File: "_parms.scm", Time-stamp: <2010-06-10 13:53:05 feeley>
 
-;;; Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved.
+;;; Copyright (c) 1994-2010 by Marc Feeley, All Rights Reserved.
 
 (include "fixnum.scm")
 
@@ -109,7 +109,16 @@
 
 (define interrupts-enabled-sym (string->canonical-symbol "interrupts-enabled"))
 
-(define environment-map-sym    (string->canonical-symbol "environment-map"))
+(define debug-sym              (string->canonical-symbol "debug"))
+(define debug-location-sym     (string->canonical-symbol "debug-location"))
+(define debug-source-sym       (string->canonical-symbol "debug-source"))
+(define debug-environments-sym (string->canonical-symbol "debug-environments"))
+
+(define environment-map-sym    (string->canonical-symbol "environment-map")) ;; deprecated: use debug-environments
+
+(define proper-tail-calls-sym  (string->canonical-symbol "proper-tail-calls"))
+
+(define optimize-dead-local-variables-sym (string->canonical-symbol "optimize-dead-local-variables"))
 
 (define generic-sym            (string->canonical-symbol "generic"))
 (define fixnum-sym             (string->canonical-symbol "fixnum"))
@@ -516,7 +525,7 @@
 
 ;; Gambit system version numbers.
 
-(define (compiler-version) 402007) ;; 100000*major + 1000*minor + revision
+(define (compiler-version) 406005) ;; 100000*major + 1000*minor + revision
 
 (define compiler-version-string-prefix "v")
 (define compiler-version-string-suffix "")
