@@ -1,11 +1,11 @@
-/* File: "main.c", Time-stamp: <2008-05-09 00:21:43 feeley> */
+/* File: "main.c", Time-stamp: <2009-08-04 13:10:04 feeley> */
 
-/* Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2009 by Marc Feeley, All Rights Reserved. */
 
 /* This is the driver of the Gambit-C system */
 
 #define ___INCLUDED_FROM_MAIN
-#define ___VERSION 402007
+#define ___VERSION 406005
 #include "gambit.h"
 
 #include "os_base.h"
@@ -41,31 +41,37 @@ int debug_settings;)
       msgs[0] =
         "Usage: program [-:OPTION,OPTION...] ...\n"
         "where OPTION is one of:\n"
-        "  mHEAPSIZE    set minimum heap size in kilobytes\n"
-        "  hHEAPSIZE    set maximum heap size in kilobytes\n"
-        "  lLIVEPERCENT set heap live ratio after GC in percent\n"
-        "  s|S          set standard Scheme mode (on|off)\n"
-        "  d[OPT...]    set debugging options; OPT is one of:\n"
-        "                 p|a       treat uncaught exceptions as errors\n"
-        "                           (primordial-thread only|all threads)\n"
-        "                 r|s|q     error handling (create a new REPL|start in\n"
-        "                           single-step mode|quit with error status)\n"
-        "                 R|D|Q     user interrupt handling (create a new REPL|defer\n"
-        "                           handling|quit with error status)\n"
-        "                 i|c|-     select REPL interaction channel\n"
-        "                           (ide|console|standard input and output)\n"
-        "                 0..9      verbosity level\n"
-        "  =DIRECTORY   override Gambit installation directory\n"
-        "  +ARGUMENT    add ARGUMENT to the command line before other arguments\n"
-        "  f[OPT...]    set file options; see below for OPT\n"
-        "  t[OPT...]    set terminal options; see below for OPT\n"
-        "  -[OPT...]    set standard input and output options; see below for OPT\n"
+        "  mHEAPSIZE       set minimum heap size in kilobytes\n"
+        "  hHEAPSIZE       set maximum heap size in kilobytes\n"
+        "  lLIVEPERCENT    set heap live ratio after GC in percent\n"
+        "  s|S             set standard Scheme mode (on|off)\n"
+        "  d[OPT...]       set debugging options; OPT is one of:\n"
+        "                    p|a       treat uncaught exceptions as errors\n"
+        "                              (primordial-thread only|all threads)\n"
+        "                    r|s|q     error handling (create a new REPL|start in\n"
+        "                              single-step mode|quit with error status)\n"
+        "                    R|D|Q     user interrupt handling (create a new REPL|\n"
+        "                              defer handling|quit with error status)\n"
+        "                    i|c|-|@[HOST][:PORT]\n"
+        "                              select REPL interaction channel (ide|console|\n"
+        "                              standard input and output|remote debugger\n"
+        "                              (defaults: HOST=127.0.0.1, PORT=44555))\n"
+        "                    0..9      verbosity level\n"
+        "  @[INTF][:PORT]  set main RPC server configuration; defaults: INTF=127.0.0.1,\n"
+        "                  PORT=44556; when INTF=* all interfaces accept connections\n"
+        "  =DIRECTORY      override central Gambit installation directory\n"
+        "  ~~DIR=DIRECTORY override Gambit installation directory ~~DIR (where DIR can\n"
+        "                  be the special \"bin\" and \"lib\", or empty, or any identifier)\n"
+        "  +ARGUMENT       add ARGUMENT to the command line before other arguments\n"
+        "  f[OPT...]       set file options; see below for OPT\n"
+        "  t[OPT...]       set terminal options; see below for OPT\n"
+        "  -[OPT...]       set standard input and output options; see below for OPT\n"
         "where OPT is one of:\n"
-        "  A|1|2|4|6|8|U  character encoding (ASCII|ISO-8859-1|UCS-2/4|UTF-16/8|UTF)\n"
-        "  l|c|cl         end-of-line encoding (LF|CR|CR-LF)\n"
-        "  u|n|f          buffering (unbuffered|newline buffered|fully buffered)\n"
-        "  r|R            enable character encoding errors (on|off)\n"
-        "  e|E            [for terminals only] enable line-editing (on|off)\n";
+        "  A|1|2|4|6|8|U   character encoding (ASCII|ISO-8859-1|UCS-2/4|UTF-16/8|UTF)\n"
+        "  l|c|cl          end-of-line encoding (LF|CR|CR-LF)\n"
+        "  u|n|f           buffering (unbuffered|newline buffered|fully buffered)\n"
+        "  r|R             enable character encoding errors (on|off)\n"
+        "  e|E             [for terminals only] enable line-editing (on|off)\n";
       msgs[1] = 0;
       ___display_error (msgs);
     }
@@ -116,43 +122,44 @@ ___UCS_2STRING *start;)
 }
 
 
-___HIDDEN ___BOOL extend_argv
-   ___P((___UCS_2STRING **argv,
+___HIDDEN ___BOOL extend_strvec
+   ___P((___UCS_2STRING **strvec,
          int pos,
          int nb_to_add,
          ___BOOL free_old),
-        (argv,
+        (strvec,
          pos,
          nb_to_add,
          free_old)
-___UCS_2STRING **argv;
+___UCS_2STRING **strvec;
 int pos;
 int nb_to_add;
 ___BOOL free_old;)
 {
   int i;
   int n = 0;
-  ___UCS_2STRING *old_argv = *argv;
-  ___UCS_2STRING *new_argv;
+  ___UCS_2STRING *old_strvec = *strvec;
+  ___UCS_2STRING *new_strvec;
 
-  while (old_argv[n++] != 0) ;
+  if (old_strvec != 0)
+    while (old_strvec[n++] != 0) ;
 
-  new_argv = ___CAST(___UCS_2STRING*,
-                     ___alloc_mem ((n+nb_to_add) * sizeof (___UCS_2STRING)));
+  new_strvec = ___CAST(___UCS_2STRING*,
+                       ___alloc_mem ((n+nb_to_add) * sizeof (___UCS_2STRING)));
 
-  if (new_argv == 0)
+  if (new_strvec == 0)
     return 0;
 
   for (i=pos; i<n; i++)
-    new_argv[i+nb_to_add] = old_argv[i];
+    new_strvec[i+nb_to_add] = old_strvec[i];
 
   for (i=0; i<pos; i++)
-    new_argv[i] = old_argv[i];
+    new_strvec[i] = old_strvec[i];
 
-  *argv = new_argv;
+  *strvec = new_strvec;
 
   if (free_old)
-    ___free_mem (old_argv);
+    ___free_mem (old_strvec);
 
   return 1;
 }
@@ -168,11 +175,18 @@ ___mod_or_lnk (*linker)();)
   ___UCS_2STRING *argv;
   ___UCS_2STRING *current_argv;
   ___UCS_2STRING runtime_options;
+  ___UCS_2STRING script_line;
   int extra_arg_pos;
   int contract_argv;
   int options_source;
+  int options_source_min;
+  int options_source_max;
   ___UCS_2STRING gambcdir;
+  ___UCS_2STRING *gambcdir_map;
+  int gambcdir_map_len;
   ___UCS_2STRING gambcopt;
+  ___UCS_2STRING remote_dbg_addr;
+  ___UCS_2STRING rpc_server_addr;
   unsigned long min_heap_len;
   unsigned long max_heap_len;
   int live_percent;
@@ -188,7 +202,11 @@ ___mod_or_lnk (*linker)();)
 
   argv = ___program_startup_info.argv;
   gambcdir = 0;
+  gambcdir_map = 0;
+  gambcdir_map_len = 0;
   gambcopt = 0;
+  remote_dbg_addr = 0;
+  rpc_server_addr = 0;
   min_heap_len = 0;
   max_heap_len = 0;
   live_percent = 0;
@@ -202,15 +220,55 @@ ___mod_or_lnk (*linker)();)
       && (runtime_options = argv[1]) != 0
       && runtime_options[0] == '-'
       && runtime_options[1] == ':')
-    runtime_options += 2;
+    {
+      runtime_options += 2;
+      if (runtime_options[0] == ':')
+        runtime_options++;
+    }
   else
     runtime_options = 0;
 
+  if ((script_line = ___program_startup_info.script_line) != 0)
+    {
+      for (;;)
+        {
+          if (*script_line == '\0')
+            {
+              script_line = 0;
+              break;
+            }
+          if (script_line[0] == ' '
+              && script_line[1] == '-'
+              && script_line[2] == ':')
+            {
+              script_line += 3;
+              break;
+            }
+          script_line++;
+        }
+    }
+
+  if (script_line != 0 &&
+      script_line[0] == ':')
+    {
+      script_line++;
+      runtime_options = 0;
+      options_source_min = 1;
+      options_source_max = 1;
+    }
+  else
+    {
+      options_source_min = 0;
+      options_source_max = 2;
+    }
+
   current_argv = argv;
   extra_arg_pos = 1;
   contract_argv = (runtime_options != 0);
 
-  for (options_source=0; options_source<3; options_source++)
+  for (options_source = options_source_min;
+       options_source <= options_source_max;
+       options_source++)
     {
       ___UCS_2STRING arg;
 
@@ -222,29 +280,7 @@ ___mod_or_lnk (*linker)();)
           arg = gambcopt;
         }
       else if (options_source == 1)
-        {
-          arg = ___program_startup_info.script_line;
-
-          if (arg != 0)
-            {
-              for (;;)
-                {
-                  if (*arg == '\0')
-                    {
-                      arg = 0;
-                      break;
-                    }
-                  if (arg[0] == ' '
-                      && arg[1] == '-'
-                      && arg[2] == ':')
-                    {
-                      arg += 3;
-                      break;
-                    }
-                  arg++;
-                }
-            }
-        }
+        arg = script_line;
       else
         arg = runtime_options;
 
@@ -309,11 +345,14 @@ ___mod_or_lnk (*linker)();)
                   while (*arg != '\0' && *arg != ' ' && *arg != ',')
                     {
                       if (*arg >= '0' && *arg <= '9')
-                        debug_settings =
-                          (debug_settings & ~___DEBUG_SETTINGS_LEVEL_MASK)
-                          | ((*arg - '0') << ___DEBUG_SETTINGS_LEVEL_SHIFT);
+                        {
+                          debug_settings =
+                            (debug_settings & ~___DEBUG_SETTINGS_LEVEL_MASK)
+                            | ((*arg - '0') << ___DEBUG_SETTINGS_LEVEL_SHIFT);
+                          arg++;
+                        }
                       else
-                        switch (*arg)
+                        switch (*arg++)
                           {
                           case 'p': debug_settings =
                                       (debug_settings
@@ -381,15 +420,42 @@ ___mod_or_lnk (*linker)();)
                                       | (___DEBUG_SETTINGS_REPL_STDIO
                                          << ___DEBUG_SETTINGS_REPL_SHIFT);
                                     break;
+
+                          case '@':
+                            debug_settings =
+                              (debug_settings
+                               & ~___DEBUG_SETTINGS_REPL_MASK)
+                              | (___DEBUG_SETTINGS_REPL_REMOTE
+                                 << ___DEBUG_SETTINGS_REPL_SHIFT);
+                            ___free_UCS_2STRING (remote_dbg_addr);
+                            remote_dbg_addr = extract_string (&arg);
+                            if (remote_dbg_addr == 0)
+                              {
+                                e = ___FIX(___HEAP_OVERFLOW_ERR);
+                                goto after_setup;
+                              }
+                            break;
+
                           default:
                             e = usage_err (debug_settings);
                             goto after_setup;
                           }
-                      arg++;
                     }
                 break;
               }
 
+            case '@':
+              {
+                ___free_UCS_2STRING (rpc_server_addr);
+                rpc_server_addr = extract_string (&arg);
+                if (rpc_server_addr == 0)
+                  {
+                    e = ___FIX(___HEAP_OVERFLOW_ERR);
+                    goto after_setup;
+                  }
+                break;
+              }
+
             case '=':
               {
                 ___free_UCS_2STRING (gambcdir);
@@ -402,14 +468,52 @@ ___mod_or_lnk (*linker)();)
                 break;
               }
 
+            case '~':
+              {
+                ___UCS_2STRING dir;
+                ___UCS_2 c;
+
+                if (*arg++ != '~')
+                  {
+                    e = usage_err (debug_settings);
+                    goto after_setup;
+                  }
+
+                s = arg;
+
+                while ((c = *s++) != '=')
+                  if (!(((c >= 'a') && (c <= 'z')) ||
+                        ((c >= 'A') && (c <= 'Z'))))
+                    {
+                      e = usage_err (debug_settings);
+                      goto after_setup;
+                    }
+
+                if (!extend_strvec (&gambcdir_map, 0, 1, gambcdir_map != 0))
+                  {
+                    e = ___FIX(___HEAP_OVERFLOW_ERR);
+                    goto after_setup;
+                  }
+
+                if ((dir = extract_string (&arg)) == 0)
+                  {
+                    e = ___FIX(___HEAP_OVERFLOW_ERR);
+                    goto after_setup;
+                  }
+
+                gambcdir_map[0] = dir;
+
+                break;
+              }
+
             case '+':
               {
                 ___UCS_2STRING extra_arg;
 
-                if (!extend_argv (&current_argv,
-                                  extra_arg_pos,
-                                  1 - contract_argv,
-                                  current_argv != argv))
+                if (!extend_strvec (&current_argv,
+                                    extra_arg_pos,
+                                    1 - contract_argv,
+                                    current_argv != argv))
                   {
                     e = ___FIX(___HEAP_OVERFLOW_ERR);
                     goto after_setup;
@@ -469,8 +573,30 @@ ___mod_or_lnk (*linker)();)
                       case '8': settings = ___CHAR_ENCODING_MASK(settings)
                                            |___CHAR_ENCODING_UTF_8;
                                 break;
-                      case 'U': settings = ___CHAR_ENCODING_MASK(settings)
-                                           |___CHAR_ENCODING_UTF;
+                      case 'U': switch (*arg++)
+                                  {
+                                  case 'A':
+                                    settings = ___CHAR_ENCODING_MASK(settings)
+                                               |___CHAR_ENCODING_UTF_FALLBACK_ASCII;
+                                    break;
+                                  case '1':
+                                    settings = ___CHAR_ENCODING_MASK(settings)
+                                               |___CHAR_ENCODING_UTF_FALLBACK_ISO_8859_1;
+                                    break;
+                                  case '6':
+                                    settings = ___CHAR_ENCODING_MASK(settings)
+                                               |___CHAR_ENCODING_UTF_FALLBACK_UTF_16;
+                                    break;
+                                  case '8':
+                                    settings = ___CHAR_ENCODING_MASK(settings)
+                                               |___CHAR_ENCODING_UTF_FALLBACK_UTF_8;
+                                    break;
+                                  default:
+                                    arg--;
+                                    settings = ___CHAR_ENCODING_MASK(settings)
+                                               |___CHAR_ENCODING_UTF;
+                                    break;
+                                  }
                                 break;
                       case 'l': settings = ___EOL_ENCODING_MASK(settings)
                                            |((___EOL_ENCODING(settings)
@@ -547,10 +673,10 @@ ___mod_or_lnk (*linker)();)
 
   if (contract_argv != 0)
     {
-      if (!extend_argv (&current_argv,
-                        extra_arg_pos,
-                        -1,
-                        0)) /* we know that current_argv == argv */
+      if (!extend_strvec (&current_argv,
+                          extra_arg_pos,
+                          -1,
+                          0)) /* we know that current_argv == argv */
         {
           e = ___FIX(___HEAP_OVERFLOW_ERR);
           goto after_setup;
@@ -577,6 +703,9 @@ ___mod_or_lnk (*linker)();)
   setup_params.terminal_settings = terminal_settings;
   setup_params.stdio_settings    = stdio_settings;
   setup_params.gambcdir          = gambcdir;
+  setup_params.gambcdir_map      = gambcdir_map;
+  setup_params.remote_dbg_addr   = remote_dbg_addr;
+  setup_params.rpc_server_addr   = rpc_server_addr;
   setup_params.linker            = linker;
 
   e = ___setup (&setup_params);
@@ -589,8 +718,16 @@ ___mod_or_lnk (*linker)();)
   if (current_argv != argv)
     ___free_mem (current_argv);
 
+  while (gambcdir_map_len > 0)
+    ___free_UCS_2STRING (gambcdir_map[gambcdir_map_len--]);
+
+  if (gambcdir_map != 0)
+    ___free_mem (gambcdir_map);
+
   ___free_UCS_2STRING (gambcdir);
   ___free_UCS_2STRING (gambcopt);
+  ___free_UCS_2STRING (remote_dbg_addr);
+  ___free_UCS_2STRING (rpc_server_addr);
 
   if (e == ___FIX(___NO_ERR))
     {
