@@ -1,6 +1,6 @@
-/* File: "os.c", Time-stamp: <2008-05-22 11:09:47 feeley> */
+/* File: "os.c" */
 
-/* Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2011 by Marc Feeley, All Rights Reserved. */
 
 /*
  * This module implements the operating system specific routines
@@ -28,7 +28,7 @@
  */
 
 #define ___INCLUDED_FROM_OS
-#define ___VERSION 402007
+#define ___VERSION 406005
 #include "gambit.h"
 
 #include "os_base.h"
@@ -58,16 +58,35 @@
 /*---------------------------------------------------------------------------*/
 
 
+___SCMOBJ ___setup_os_interrupt_handling ___PVOID
+{
+  ___SCMOBJ e;
+
+  if ((e = ___setup_heartbeat_interrupt_handling ()) == ___FIX(___NO_ERR))
+    {
+      if ((e = ___setup_user_interrupt_handling ()) != ___FIX(___NO_ERR))
+        ___cleanup_heartbeat_interrupt_handling ();
+    }
+
+  return e;
+}
+
+void ___cleanup_os_interrupt_handling ___PVOID
+{
+  ___cleanup_user_interrupt_handling ();
+  ___cleanup_heartbeat_interrupt_handling ();
+}
+
 void ___disable_os_interrupts ___PVOID
 {
-  ___disable_heartbeat_interrupt ();
-  ___disable_user_interrupt ();
+  ___disable_heartbeat_interrupts ();
+  ___disable_user_interrupts ();
 }
 
 void ___enable_os_interrupts ___PVOID
 {
-  ___enable_user_interrupt ();
-  ___enable_heartbeat_interrupt ();
+  ___enable_user_interrupts ();
+  ___enable_heartbeat_interrupts ();
 }
 
 
@@ -149,6 +168,149 @@ ___BOOL pinpoint;)
 #define USE_IPV6
 #endif
 
+
+___HIDDEN int network_family_decode
+   ___P((int family),
+        (family)
+int family;)
+{
+  switch (family)
+    {
+#ifdef PF_INET
+    case -1:
+      return PF_INET;
+#endif
+
+#ifdef PF_INET6
+    case -2:
+      return PF_INET6;
+#endif
+    }
+
+  return 0;
+}
+
+
+___HIDDEN ___SCMOBJ network_family_encode
+   ___P((int family),
+        (family)
+int family;)
+{
+  switch (family)
+    {
+#ifdef PF_INET
+    case PF_INET:
+      return ___FIX(-1);
+#endif
+
+#ifdef PF_INET6
+    case PF_INET6:
+      return ___FIX(-2);
+#endif
+    }
+
+  return ___FIX(family);
+}
+
+
+___HIDDEN int network_socktype_decode
+   ___P((int socktype),
+        (socktype)
+int socktype;)
+{
+  switch (socktype)
+    {
+#ifdef SOCK_STREAM
+    case -1:
+      return SOCK_STREAM;
+#endif
+
+#ifdef SOCK_DGRAM
+    case -2:
+      return SOCK_DGRAM;
+#endif
+
+#ifdef SOCK_RAW
+    case -3:
+      return SOCK_RAW;
+#endif
+    }
+
+  return 0;
+}
+
+
+___HIDDEN ___SCMOBJ network_socktype_encode
+   ___P((int socktype),
+        (socktype)
+int socktype;)
+{
+  switch (socktype)
+    {
+#ifdef SOCK_STREAM
+    case SOCK_STREAM:
+      return ___FIX(-1);
+#endif
+
+#ifdef SOCK_DGRAM
+    case SOCK_DGRAM:
+      return ___FIX(-2);
+#endif
+
+#ifdef SOCK_RAW
+    case SOCK_RAW:
+      return ___FIX(-3);
+#endif
+    }
+
+  return ___FIX(socktype);
+}
+
+
+___HIDDEN int network_protocol_decode
+   ___P((int protocol),
+        (protocol)
+int protocol;)
+{
+  switch (protocol)
+    {
+#ifdef IPPROTO_UDP
+    case -1:
+      return IPPROTO_UDP;
+#endif
+
+#ifdef IPPROTO_TCP
+    case -2:
+      return IPPROTO_TCP;
+#endif
+    }
+
+  return 0;
+}
+
+
+___HIDDEN ___SCMOBJ network_protocol_encode
+   ___P((int protocol),
+        (protocol)
+int protocol;)
+{
+  switch (protocol)
+    {
+#ifdef IPPROTO_UDP
+    case IPPROTO_UDP:
+      return ___FIX(-1);
+#endif
+
+#ifdef IPPROTO_TCP
+    case IPPROTO_TCP:
+      return ___FIX(-2);
+#endif
+    }
+
+  return ___FIX(protocol);
+}
+
+
 ___SCMOBJ ___SCMOBJ_to_in_addr
    ___P((___SCMOBJ addr,
          struct in_addr *ia,
@@ -358,7 +520,7 @@ int arg_num;)
           return addr;
         }
 
-      ___FIELD(result,1) = ___FIX(sa_in->sin_family);
+      ___FIELD(result,1) = network_family_encode (sa_in->sin_family);
       ___FIELD(result,2) = ___FIX(ntohs (sa_in->sin_port));
       ___FIELD(result,3) = addr;
       ___release_scmobj (addr);
@@ -375,7 +537,7 @@ int arg_num;)
           return addr;
         }
 
-      ___FIELD(result,1) = ___FIX(sa_in6->sin6_family);
+      ___FIELD(result,1) = network_family_encode (sa_in6->sin6_family);
       ___FIELD(result,2) = ___FIX(ntohs (sa_in6->sin6_port));
       ___FIELD(result,3) = addr;
       ___release_scmobj (addr);
@@ -453,6 +615,193 @@ int arg_num;)
 
 /* Access to host information. */
 
+#ifdef USE_getaddrinfo
+
+___HIDDEN int ai_flags_decode
+   ___P((int flags),
+        (flags)
+int flags;)
+{
+  int ai_flags = 0;
+
+#ifdef AI_PASSIVE
+  if (flags & 1)
+    ai_flags |= AI_PASSIVE;
+#endif
+
+#ifdef AI_CANONNAME
+  if (flags & 2)
+    ai_flags |= AI_CANONNAME;
+#endif
+
+#ifdef AI_NUMERICHOST
+  if (flags & 4)
+    ai_flags |= AI_NUMERICHOST;
+#endif
+
+#ifdef AI_NUMERICSERV
+  if (flags & 8)
+    ai_flags |= AI_NUMERICSERV;
+#endif
+
+#ifdef AI_ALL
+  if (flags & 16)
+    ai_flags |= AI_ALL;
+#endif
+
+#ifdef AI_ADDRCONFIG
+  if (flags & 32)
+    ai_flags |= AI_ADDRCONFIG;
+#endif
+
+#ifdef AI_V4MAPPED
+  if (flags & 64)
+    ai_flags |= AI_V4MAPPED;
+#endif
+
+  return ai_flags;
+}
+
+#endif
+
+
+___SCMOBJ ___os_address_infos
+   ___P((___SCMOBJ host,
+         ___SCMOBJ serv,
+         ___SCMOBJ flags,
+         ___SCMOBJ family,
+         ___SCMOBJ socktype,
+         ___SCMOBJ protocol),
+        (host,
+         serv,
+         flags,
+         family,
+         socktype,
+         protocol)
+___SCMOBJ host;
+___SCMOBJ serv;
+___SCMOBJ flags;
+___SCMOBJ family;
+___SCMOBJ socktype;
+___SCMOBJ protocol;)
+{
+#ifndef USE_getaddrinfo
+
+  return ___FIX(___UNIMPL_ERR);
+
+#endif
+
+#ifdef USE_getaddrinfo
+
+  ___SCMOBJ e;
+  ___SCMOBJ vect;
+  ___SCMOBJ lst;
+  ___SCMOBJ tail;
+  ___SCMOBJ x;
+  ___SCMOBJ p;
+  int i;
+  char *chost = 0;
+  char *cserv = 0;
+
+  struct addrinfo hints, *res, *res0;
+  int code;
+
+  if ((e = ___SCMOBJ_to_CHARSTRING (host, &chost, 1))
+      != ___FIX(___NO_ERR))
+    return e;
+
+  if ((e = ___SCMOBJ_to_CHARSTRING (serv, &cserv, 2))
+      != ___FIX(___NO_ERR))
+    {
+      ___release_string (chost);
+      return e;
+    }
+
+  memset (&hints, 0, sizeof (hints));
+
+  hints.ai_flags    = ai_flags_decode (___INT(flags));
+  hints.ai_family   = network_family_decode (___INT(family));
+  hints.ai_socktype = network_socktype_decode (___INT(socktype));
+  hints.ai_protocol = network_protocol_decode (___INT(protocol));
+
+  code = getaddrinfo (chost, cserv, &hints, &res0);
+
+  if (code != 0)
+    {
+      e = err_code_from_gai_code (code);
+      ___release_string (chost);
+      ___release_string (cserv);
+      return e;
+    }
+
+  lst = ___NUL;
+  tail = ___FAL;
+
+  for (res = res0; res != NULL; res = res->ai_next)
+    {
+      x = ___sockaddr_to_SCMOBJ (res->ai_addr,
+                                 res->ai_addrlen,
+                                 ___RETURN_POS);
+
+      if (___FIXNUMP(x))
+        {
+          ___release_scmobj (lst);
+          freeaddrinfo (res0);
+          return x;
+        }
+
+      if (x != ___FAL)
+        {
+          vect = ___make_vector (5, ___FAL, ___STILL);
+
+          if (___FIXNUMP(vect))
+            {
+              ___release_scmobj (x);
+              ___release_scmobj (lst);
+              freeaddrinfo (res0);
+              return ___FIX(___CTOS_HEAP_OVERFLOW_ERR+___RETURN_POS);
+            }
+
+          ___FIELD(vect,1) = network_family_encode (res->ai_family);
+          ___FIELD(vect,2) = network_socktype_encode (res->ai_socktype);
+          ___FIELD(vect,3) = network_protocol_encode (res->ai_protocol);
+          ___FIELD(vect,4) = x;
+
+          ___release_scmobj (x);
+
+          p = ___make_pair (vect, ___NUL, ___STILL);
+
+          ___release_scmobj (vect);
+
+          if (___FIXNUMP(p))
+            {
+              ___release_scmobj (lst);
+              freeaddrinfo (res0);
+              return ___FIX(___CTOS_HEAP_OVERFLOW_ERR+___RETURN_POS);
+            }
+
+          if (lst == ___NUL)
+            lst = p;
+          else
+            ___SETCDR(tail,p);
+
+          tail = p;
+        }
+    }
+
+  ___release_scmobj (lst);
+
+  freeaddrinfo (res0);
+
+  ___release_string (chost);
+  ___release_string (cserv);
+
+  return lst;
+
+#endif
+}
+
+
 ___SCMOBJ ___os_host_info
    ___P((___SCMOBJ host),
         (host)
@@ -1062,7 +1411,64 @@ ___SCMOBJ chase;)
 #ifndef USE_stat
 #ifndef USE_GetFileAttributesEx
 
-  return ___FIX(___UNIMPL_ERR);
+#define ___INFO_PATH_CE_SELECT(latin1,utf8,ucs2,ucs4,wchar,native) native
+
+  if ((e = ___SCMOBJ_to_NONNULLSTRING
+             (path,
+              &cpath,
+              1,
+              ___CE(___INFO_PATH_CE_SELECT),
+              0))
+      == ___FIX(___NO_ERR))
+    {
+      ___FILE *check_exist = ___fopen (cpath, "r");
+
+      if (check_exist == 0)
+        {
+          e = fnf_or_err_code_from_errno ();
+          ___release_string (cpath);
+          return e;
+        }
+
+      ___fclose (check_exist);
+
+      ___release_string (cpath);
+
+      result = ___make_vector (14, ___FAL, ___STILL);
+
+      if (___FIXNUMP(result))
+        return ___FIX(___CTOS_HEAP_OVERFLOW_ERR+___RETURN_POS);/**********/
+
+      ___FIELD(result,1) = ___FIX(0); /* unknown type */
+      ___FIELD(result,2) = ___FIX(0);
+      ___FIELD(result,3) = ___FIX(0);
+      ___FIELD(result,4) = ___FIX(0);
+      ___FIELD(result,5) = ___FIX(0);
+      ___FIELD(result,6) = ___FIX(0);
+      ___FIELD(result,7) = ___FIX(0);
+      ___FIELD(result,8) = ___FIX(0);
+
+      if ((e = ___F64_to_SCMOBJ (___CAST(___F64,NEG_INFINITY),
+                                 &x,
+                                 ___RETURN_POS))
+          != ___FIX(___NO_ERR))
+        {
+          ___release_scmobj (result);
+          return e;
+        }
+
+      ___FIELD(result,9) = x;
+      ___FIELD(result,10) = x;
+      ___FIELD(result,11) = x;
+      ___FIELD(result,12) = ___FIX(0);
+      ___FIELD(result,13) = x;
+
+      ___release_scmobj (x);
+
+      ___release_scmobj (result);
+
+      return result;
+    }
 
 #endif
 #endif
@@ -1079,11 +1485,11 @@ ___SCMOBJ chase;)
               0))
       == ___FIX(___NO_ERR))
     {
-      struct stat s;
+      ___struct_stat s;
 
       if (((chase == ___FAL)
-           ? lstat (___CAST(___STRING_TYPE(___INFO_PATH_CE_SELECT),cpath), &s)
-           : stat (___CAST(___STRING_TYPE(___INFO_PATH_CE_SELECT),cpath), &s))
+           ? ___lstat (___CAST(___STRING_TYPE(___INFO_PATH_CE_SELECT),cpath), &s)
+           : ___stat (___CAST(___STRING_TYPE(___INFO_PATH_CE_SELECT),cpath), &s))
           < 0)
         {
           e = fnf_or_err_code_from_errno ();
@@ -1671,6 +2077,10 @@ ___SCMOBJ ___os_getppid ___PVOID
 #define ___SYS_TYPE_OS "unknown"
 #endif
 
+#ifndef ___CONFIGURE_COMMAND
+#define ___CONFIGURE_COMMAND "unknown"
+#endif
+
 
 ___HIDDEN char *os_sys_type[] =
 { ___SYS_TYPE_CPU, ___SYS_TYPE_VENDOR, ___SYS_TYPE_OS, NULL };
@@ -1680,6 +2090,9 @@ ___HIDDEN char *os_sys_type_string =
 ___SYS_TYPE_CPU "-" ___SYS_TYPE_VENDOR "-" ___SYS_TYPE_OS;
 
 
+___HIDDEN char *configure_command_string = ___CONFIGURE_COMMAND;
+
+
 char **___os_system_type ___PVOID
 {
   return os_sys_type;
@@ -1692,6 +2105,55 @@ char *___os_system_type_string ___PVOID
 }
 
 
+char *___os_configure_command_string ___PVOID
+{
+  return configure_command_string;
+}
+
+
+/*---------------------------------------------------------------------------*/
+
+/* C compilation environment information. */
+
+
+#ifndef ___OBJ_EXTENSION
+#define ___OBJ_EXTENSION ".obj"
+#endif
+
+#ifndef ___EXE_EXTENSION
+#define ___EXE_EXTENSION ".exe"
+#endif
+
+#ifndef ___BAT_EXTENSION
+#define ___BAT_EXTENSION ".bat"
+#endif
+
+
+___HIDDEN char *os_obj_extension_string = ___OBJ_EXTENSION;
+
+___HIDDEN char *os_exe_extension_string = ___EXE_EXTENSION;
+
+___HIDDEN char *os_bat_extension_string = ___BAT_EXTENSION;
+
+
+char *___os_obj_extension_string ___PVOID
+{
+  return os_obj_extension_string;
+}
+
+
+char *___os_exe_extension_string ___PVOID
+{
+  return os_exe_extension_string;
+}
+
+
+char *___os_bat_extension_string ___PVOID
+{
+  return os_bat_extension_string;
+}
+
+
 /*---------------------------------------------------------------------------*/
 
 
