@@ -1,13 +1,13 @@
-/* File: "os_base.c", Time-stamp: <2008-05-17 08:22:56 feeley> */
+/* File: "os_base.c" */
 
-/* Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2012 by Marc Feeley, All Rights Reserved. */
 
 /*
  * This module implements the most basic operating system services.
  */
 
 #define ___INCLUDED_FROM_OS_BASE
-#define ___VERSION 402007
+#define ___VERSION 406005
 #include "gambit.h"
 
 #include "os_base.h"
@@ -117,10 +117,8 @@ ___FILE *stream;)
 int ___printf
    ___P((const char *format,
          ...),
-        (format,
-         va_alist)
-const char *format;
-va_dcl)
+        (format, ...)
+const char *format;)
 {
   va_list ap;
   int result;
@@ -186,19 +184,6 @@ unsigned long bytes;)
 
   ptr = malloc (bytes);
 
-#endif
-
-#ifdef ___DEBUG
-#ifdef ___DEBUG_ALLOC_MEM_TRACE
-  if (___base_mod.file != 0)
-    ___printf ("%p (%lu bytes) ALLOCATED AT \"%s\"@%d.1\n",
-               ptr,
-               bytes,
-               ___base_mod.file,
-               ___base_mod.lineno);
-  else
-    ___printf ("%p (%lu bytes) ALLOCATED\n", ptr, bytes);
-#endif
 #endif
 
   return ptr;
@@ -265,13 +250,16 @@ char *file;)
 {
   void *ptr;
 
-  ___base_mod.lineno = lineno;
-  ___base_mod.file = file;
-
   ptr = ___alloc_mem (bytes);
 
-  ___base_mod.lineno = 0;
-  ___base_mod.file = 0;
+  if (file != 0)
+    ___printf ("%p (%lu bytes) ALLOCATED AT \"%s\"@%d.1\n",
+               ptr,
+               bytes,
+               file,
+               lineno);
+  else
+    ___printf ("%p (%lu bytes) ALLOCATED\n", ptr, bytes);
 
   return ptr;
 }
@@ -281,15 +269,102 @@ char *file;)
 #endif
 
 
+void *___alloc_mem_code
+   ___P((unsigned long bytes),
+        (bytes)
+unsigned long bytes;)
+{
+#ifndef USE_mmap
+#ifndef USE_VirtualAlloc
+
+  return NULL;
+
+#endif
+#endif
+
+#ifdef USE_mmap
+
+  ___BOOL executable = 1;
+
+  void* ptr = mmap (0,
+                    bytes + sizeof (long),
+                    PROT_READ | PROT_WRITE | (executable ? PROT_EXEC : 0),
+                    MAP_PRIVATE | MAP_ANON,
+                    -1,
+                    0);
+
+  if (ptr == MAP_FAILED)
+    return NULL;
+
+  *___CAST(long*,ptr) = bytes;
+
+  return ___CAST(long*,ptr)+1;
+
+#endif
+
+#ifdef USE_VirtualAlloc
+
+  ___BOOL executable = 1;
+
+  void *ptr = VirtualAlloc (NULL,
+                            bytes,
+                            MEM_COMMIT,
+                            (executable
+                             ? PAGE_EXECUTE_READWRITE
+                             : PAGE_READWRITE));
+
+  return ptr;
+
+#endif
+}
+
+
+void ___free_mem_code
+   ___P((void *ptr),
+        (ptr)
+void *ptr;)
+{
+#ifndef USE_mmap
+#ifndef USE_VirtualAlloc
+
+#endif
+#endif
+
+#ifdef USE_mmap
+
+  long* p = ___CAST(long*,ptr)-1;
+
+  munmap (p, *p + sizeof (long));
+
+#endif
+
+#ifdef USE_VirtualAlloc
+
+  VirtualFree (ptr, 0, MEM_RELEASE);
+
+#endif
+}
+
+
 /*---------------------------------------------------------------------------*/
 
 /* Program startup */
 
 
+/*
+ * ___main_char, ___main_UCS_2, and ___winmain are variants of main
+ * entry points which differ in the format of the arguments; the first
+ * two are always compiled in since they could be useful on all
+ * systems for embedding of Gambit.
+ */
+
+
+/* To keep command line and runtime flag information around: */
+
 ___program_startup_info_struct ___program_startup_info =
 {
-  0,
-  0
+  0, /* argument vector */
+  0  /* runtime flag string */
 
 #ifdef ___OS_WIN32
   ,
@@ -838,6 +913,61 @@ ___SCMOBJ ___err_code_from_h_errno ___PVOID
 #endif
 
 
+#ifdef USE_getaddrinfo
+
+
+___HIDDEN const char *gai_code_to_string
+   ___P((int code),
+        (code)
+int code;)
+{
+  return gai_strerror (code);
+}
+
+
+#ifdef ___DEBUG
+___SCMOBJ ___err_code_from_gai_code_debug
+   ___P((int code,
+         int lineno,
+         char *file),
+        (code,
+         lineno,
+         file)
+int code;
+int lineno;
+char *file;)
+#else
+___SCMOBJ ___err_code_from_gai_code
+   ___P((int code),
+        (code)
+int code;)
+#endif
+{
+  ___ERR_CODE e;
+
+#ifdef EAI_SYSTEM
+  if (code == EAI_SYSTEM)
+    e = err_code_from_errno ();
+  else
+#endif
+
+    e = ___GAI_CODE_ERR(code);
+
+#ifdef ___DEBUG
+  ___printf ("*** OS ERROR AT \"%s\"@%d.1 -- gai_code=%d (%s)\n",
+             file,
+             lineno,
+             code,
+             gai_code_to_string (code));
+#endif
+
+  return ___FIX(e);
+}
+
+
+#endif
+
+
 #ifdef USE_GetLastError
 
 
@@ -1211,6 +1341,21 @@ ___SCMOBJ err;)
 
       append_charstring (buf, &pos, msg);
 
+#endif
+    }
+  else if (facility >= ___ERR_CODE_FACILITY_GAI_CODE)
+    {
+      /* getaddressinfo error code */
+
+#ifdef USE_getaddrinfo
+
+      const char *msg = gai_code_to_string (___GAI_CODE_FROM_ERR_CODE(err_code));
+
+      if (msg == NULL)
+        msg = "Unknown error";
+
+      append_charstring (buf, &pos, msg);
+
 #endif
     }
   else
@@ -1439,7 +1584,7 @@ ___SCMOBJ ___setup_base_module ___PVOID
 #endif
 
 #ifdef USE_WIN32
-      ___base_mod.debug = ___fopen ("console", "w");
+      ___base_mod.debug = ___fopen ("con:", "w");
 #endif
 
       if (___base_mod.debug == NULL)
@@ -1450,13 +1595,6 @@ ___SCMOBJ ___setup_base_module ___PVOID
       ___base_mod.alloc_mem_calls = 0;
       ___base_mod.free_mem_calls = 0;
 
-#ifdef ___DEBUG_ALLOC_MEM_TRACE
-
-      ___base_mod.lineno = 0;
-      ___base_mod.file = 0;
-
-#endif
-
 #endif
 
       setup_fp ();
