@@ -1,6 +1,6 @@
-/* File: "os_dyn.c", Time-stamp: <2007-09-11 23:51:19 feeley> */
+/* File: "os_dyn.c", Time-stamp: <2009-06-07 10:55:37 feeley> */
 
-/* Copyright (c) 1994-2007 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved. */
 
 /*
  * This module implements the operating system specific routines
@@ -8,7 +8,7 @@
  */
 
 #define ___INCLUDED_FROM_OS_DYN
-#define ___VERSION 402007
+#define ___VERSION 406005
 #include "gambit.h"
 
 #include "os_base.h"
@@ -370,12 +370,46 @@ void **linker;)
 
 #endif
 
+  if (result != ___FIX(___NO_ERR))
+    {
+      ___SCMOBJ r = ___make_vector (2, ___FAL, ___STILL);
+
+      if (!___FIXNUMP(r))
+        {
+          ___SCMOBJ modname;
+
+          if ((e = ___NONNULLSTRING_to_SCMOBJ
+                     (cmodname,
+                      &modname,
+                      ___RETURN_POS,
+                      ___CE(___DL_MODNAME_CE_SELECT)))
+              != ___FIX(___NO_ERR))
+            ___release_scmobj (r);
+          else
+            {
+              ___VECTORSET(r,___FIX(0),result)
+              ___VECTORSET(r,___FIX(1),modname)
+              ___release_scmobj (result);
+              ___release_scmobj (modname);
+              result = r;
+            }
+        }
+    }
+
   return result;
 }
 
 
 ___HIDDEN void cleanup_dynamic_load ___PVOID
 {
+#ifndef ___DONT_UNLOAD_DYN_CODE
+
+/*
+ * When the --fprofile-arcs option of gcc is used, shared libraries
+ * must not be unloaded, otherwise a segmentation fault occurs on
+ * program exit (specifically when gcov_exit is called).
+ */
+
 #ifdef ___DL_DESCR
 
   ___dl_entry *p = ___dyn_mod.dl_list;
@@ -416,6 +450,8 @@ ___HIDDEN void cleanup_dynamic_load ___PVOID
 
   ___dyn_mod.dl_list = 0;
 
+#endif
+
 #endif
 }
 
