@@ -1,6 +1,6 @@
-/* File: "os_time.c", Time-stamp: <2007-09-11 23:51:43 feeley> */
+/* File: "os_time.c", Time-stamp: <2011-03-22 14:38:49 feeley> */
 
-/* Copyright (c) 1994-2007 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2008 by Marc Feeley, All Rights Reserved. */
 
 /*
  * This module implements the operating system specific routines
@@ -8,7 +8,7 @@
  */
 
 #define ___INCLUDED_FROM_OS_TIME
-#define ___VERSION 402007
+#define ___VERSION 406005
 #include "gambit.h"
 
 #include "os_base.h"
@@ -447,6 +447,15 @@ ___time tim;)
 #ifdef USE_select
 
 
+#ifdef ___TIMEVAL_NOT_LIMITED
+#define ___TIMEVAL_SEC_LIMIT 2147483647 /* in seconds = 68 years */
+#else
+/* Mac OS X gives an error when the seconds > 100000000 (3.2 years) */
+/* We'll be conservative in case other systems have limits */
+#define ___TIMEVAL_SEC_LIMIT 9999999 /* in seconds = 118 days */
+#endif
+
+
 void ___absolute_time_to_nonnegative_timeval
    ___P((___time tim,
          struct timeval **tv),
@@ -462,9 +471,9 @@ struct timeval **tv;)
       if (___time_positive (tim))
         {
 #ifdef ___FLOAT_TIME_REPRESENTATION
-          if (tim >= 2147483648.0) /* upper bound is 68 years! */
+          if (tim >= (___TIMEVAL_SEC_LIMIT+1.0))
             {
-              t->tv_sec = 2147483647;
+              t->tv_sec = ___TIMEVAL_SEC_LIMIT;
               t->tv_usec = 999999;
             }
           else
@@ -475,8 +484,16 @@ struct timeval **tv;)
 #endif
 
 #ifdef ___INT_TIME_REPRESENTATION
-          t->tv_sec = tim.secs;
-          t->tv_usec = tim.nsecs / 1000;
+          if (tim.secs > ___TIMEVAL_SEC_LIMIT)
+            {
+              t->tv_sec = ___TIMEVAL_SEC_LIMIT;
+              t->tv_usec = 999999;
+            }
+          else
+            {
+              t->tv_sec = tim.secs;
+              t->tv_usec = tim.nsecs / 1000;
+            }
 #endif
         }
       else
@@ -713,31 +730,6 @@ ___HIDDEN void cleanup_process_times ___PVOID
 /* Heartbeat interrupt handling. */
 
 
-/*
- * Use the process-time timer unless only the real-time timer is
- * available (e.g. DJGPP).  Note that on some systems (e.g. MkLinux)
- * ITIMER_VIRTUAL is an enum type, not a macro.
- */
-
-#ifdef __CYGWIN__
-/* ITIMER_VIRTUAL is broken under cygwin... use ITIMER_REAL */
-#undef ITIMER_VIRTUAL
-#endif
-
-#ifdef ITIMER_VIRTUAL
-#define HEARTBEAT_ITIMER ITIMER_VIRTUAL
-#define HEARTBEAT_SIG SIGVTALRM
-#else
-#ifdef ITIMER_REAL
-#define HEARTBEAT_ITIMER ITIMER_REAL
-#define HEARTBEAT_SIG SIGALRM
-#else
-#define HEARTBEAT_ITIMER ITIMER_VIRTUAL
-#define HEARTBEAT_SIG SIGVTALRM
-#endif
-#endif
-
-
 #ifndef USE_setitimer
 #ifndef USE_dos_setvect_1Ch
 #ifndef USE_DosStartTimer
@@ -755,23 +747,23 @@ ___F64 seconds;)
 }
 
 
-void ___disable_heartbeat_interrupt ___PVOID
+void ___disable_heartbeat_interrupts ___PVOID
 {
 }
 
 
-void ___enable_heartbeat_interrupt ___PVOID
+void ___enable_heartbeat_interrupts ___PVOID
 {
 }
 
 
-___HIDDEN ___SCMOBJ setup_heartbeat_interrupt_handling ___PVOID
+___SCMOBJ ___setup_heartbeat_interrupt_handling ___PVOID
 {
   return ___FIX(___NO_ERR);
 }
 
 
-___HIDDEN void cleanup_heartbeat_interrupt_handling ___PVOID
+void ___cleanup_heartbeat_interrupt_handling ___PVOID
 {
 }
 
@@ -849,7 +841,7 @@ ___F64 seconds;)
 }
 
 
-void ___disable_heartbeat_interrupt ___PVOID
+void ___disable_heartbeat_interrupts ___PVOID
 {
   ___F64 save_heartbeat_interval = ___time_mod.current_heartbeat_interval;
   ___set_heartbeat_interval (-1.0);
@@ -860,7 +852,7 @@ void ___disable_heartbeat_interrupt ___PVOID
 }
 
 
-void ___enable_heartbeat_interrupt ___PVOID
+void ___enable_heartbeat_interrupts ___PVOID
 {
   ___set_heartbeat_interval (___time_mod.current_heartbeat_interval);
 #ifdef USE_POSIX
@@ -869,16 +861,16 @@ void ___enable_heartbeat_interrupt ___PVOID
 }
 
 
-___HIDDEN ___SCMOBJ setup_heartbeat_interrupt_handling ___PVOID
+___SCMOBJ ___setup_heartbeat_interrupt_handling ___PVOID
 {
-  ___enable_heartbeat_interrupt ();
+  ___enable_heartbeat_interrupts ();
   return ___FIX(___NO_ERR);
 }
 
 
-___HIDDEN void cleanup_heartbeat_interrupt_handling ___PVOID
+void ___cleanup_heartbeat_interrupt_handling ___PVOID
 {
-  ___disable_heartbeat_interrupt ();
+  ___disable_heartbeat_interrupts ();
 }
 
 
@@ -939,19 +931,19 @@ ___F64 seconds;)
 }
 
 
-void ___disable_heartbeat_interrupt ___PVOID
+void ___disable_heartbeat_interrupts ___PVOID
 {
   ___time_mod.heartbeat_enabled = 0;
 }
 
 
-void ___enable_heartbeat_interrupt ___PVOID
+void ___enable_heartbeat_interrupts ___PVOID
 {
   ___time_mod.heartbeat_enabled = 1;
 }
 
 
-___HIDDEN ___SCMOBJ setup_heartbeat_interrupt_handling ___PVOID
+___SCMOBJ ___setup_heartbeat_interrupt_handling ___PVOID
 {
   prev_vector_1Ch = _dos_getvect (0x1C);
   ___time_mod.heartbeat_enabled = 1;
@@ -960,7 +952,7 @@ ___HIDDEN ___SCMOBJ setup_heartbeat_interrupt_handling ___PVOID
 }
 
 
-___HIDDEN void cleanup_heartbeat_interrupt_handling ___PVOID
+void ___cleanup_heartbeat_interrupt_handling ___PVOID
 {
   ___set_heartbeat_interval (-1.0);
   ___time_mod.heartbeat_enabled = 0;
@@ -1035,19 +1027,19 @@ ___F64 seconds;)
 }
 
 
-void ___disable_heartbeat_interrupt ___PVOID
+void ___disable_heartbeat_interrupts ___PVOID
 {
   ___time_mod.heartbeat_enabled = 0;
 }
 
 
-void ___enable_heartbeat_interrupt ___PVOID
+void ___enable_heartbeat_interrupts ___PVOID
 {
   ___time_mod.heartbeat_enabled = 1;
 }
 
 
-___HIDDEN ___SCMOBJ setup_heartbeat_interrupt_handling ___PVOID
+___SCMOBJ ___setup_heartbeat_interrupt_handling ___PVOID
 {
   ___time_mod.heartbeat_enabled = 0;
   ___time_mod.heartbeat_hev     = 0;
@@ -1071,7 +1063,7 @@ ___HIDDEN ___SCMOBJ setup_heartbeat_interrupt_handling ___PVOID
 }
 
 
-___HIDDEN void cleanup_heartbeat_interrupt_handling ___PVOID
+void ___cleanup_heartbeat_interrupt_handling ___PVOID
 {
   ___time_mod.heartbeat_enabled = 0;
   if (___time_mod.heartbeat_htimer)
@@ -1176,19 +1168,19 @@ ___F64 seconds;)
 }
 
 
-void ___disable_heartbeat_interrupt ___PVOID
+void ___disable_heartbeat_interrupts ___PVOID
 {
   ___time_mod.heartbeat_enabled = 0;
 }
 
 
-void ___enable_heartbeat_interrupt ___PVOID
+void ___enable_heartbeat_interrupts ___PVOID
 {
   ___time_mod.heartbeat_enabled = 1;
 }
 
 
-___HIDDEN ___SCMOBJ setup_heartbeat_interrupt_handling ___PVOID
+___SCMOBJ ___setup_heartbeat_interrupt_handling ___PVOID
 {
   ___time_mod.heartbeat_enabled = 1;
   ___time_mod.heartbeat_task_installed = 0;
@@ -1196,7 +1188,7 @@ ___HIDDEN ___SCMOBJ setup_heartbeat_interrupt_handling ___PVOID
 }
 
 
-___HIDDEN void cleanup_heartbeat_interrupt_handling ___PVOID
+void ___cleanup_heartbeat_interrupt_handling ___PVOID
 {
   ___set_heartbeat_interval (-1.0);
   ___time_mod.heartbeat_enabled = 0;
@@ -1287,19 +1279,19 @@ ___F64 seconds;)
 }
 
 
-void ___disable_heartbeat_interrupt ___PVOID
+void ___disable_heartbeat_interrupts ___PVOID
 {
   ___time_mod.heartbeat_enabled = 0;
 }
 
 
-void ___enable_heartbeat_interrupt ___PVOID
+void ___enable_heartbeat_interrupts ___PVOID
 {
   ___time_mod.heartbeat_enabled = 1;
 }
 
 
-___HIDDEN ___SCMOBJ setup_heartbeat_interrupt_handling ___PVOID
+___SCMOBJ ___setup_heartbeat_interrupt_handling ___PVOID
 {
   ___SCMOBJ e;
   DWORD thread_id;
@@ -1349,7 +1341,7 @@ ___HIDDEN ___SCMOBJ setup_heartbeat_interrupt_handling ___PVOID
 }
 
 
-___HIDDEN void cleanup_heartbeat_interrupt_handling ___PVOID
+void ___cleanup_heartbeat_interrupt_handling ___PVOID
 {
   if (___time_mod.heartbeat_thread != NULL)
     {
@@ -1383,7 +1375,7 @@ void (*heartbeat_interrupt_handler) ___PVOID;)
     {
       ___time_mod.heartbeat_interrupt_handler = heartbeat_interrupt_handler;
       setup_process_times ();
-      if ((e = setup_heartbeat_interrupt_handling ()) != ___FIX(___NO_ERR))
+      if ((e = ___setup_heartbeat_interrupt_handling ()) != ___FIX(___NO_ERR))
         return e;
       ___time_mod.setup = 1;
       return ___FIX(___NO_ERR);
@@ -1397,7 +1389,7 @@ void ___cleanup_time_module ___PVOID
 {
   if (___time_mod.setup)
     {
-      cleanup_heartbeat_interrupt_handling ();
+      ___cleanup_heartbeat_interrupt_handling ();
       cleanup_process_times ();
       ___time_mod.setup = 0;
     }
