@@ -1,6 +1,6 @@
-/* File: "setup.c", Time-stamp: <2008-02-13 00:17:13 feeley> */
+/* File: "setup.c" */
 
-/* Copyright (c) 1994-2007 by Marc Feeley, All Rights Reserved. */
+/* Copyright (c) 1994-2012 by Marc Feeley, All Rights Reserved. */
 
 /* 
  * This module contains the routines that setup the Scheme program for
@@ -8,7 +8,7 @@
  */
 
 #define ___INCLUDED_FROM_SETUP
-#define ___VERSION 402007
+#define ___VERSION 406005
 #include "gambit.h"
 
 #include "os_base.h"
@@ -43,7 +43,7 @@ ___HIDDEN ___UCS_2 reset_argv0[] = { 0 };
 ___HIDDEN ___UCS_2STRING reset_argv[] = { reset_argv0, 0 };
 
 ___setup_params_struct ___setup_params =
-{ 0, reset_argv, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
+{ 0, reset_argv, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0 };
 
 
 /* 
@@ -995,10 +995,36 @@ ___module_struct *module;)
 {
   if (module->lbl_count > 0)
     {
-      ___FIELD(___FIELD(module_descr,0),module_count) =
-        *module->lp+___LS*___WS;
+      ___SCMOBJ err;
+      ___SCMOBJ mod_name;
+      ___SCMOBJ descr = ___make_vector (2, ___FAL, ___STILL);
+
+      if (___FIXNUMP(descr))
+        return descr;
+
+      if ((err = ___NONNULLUTF_8STRING_to_SCMOBJ
+                   (module->name+1,
+                    &mod_name,
+                    0))
+          != ___FIX(___NO_ERR))
+        {
+          ___release_scmobj (descr);
+          return err;
+        }
+
+      ___FIELD(descr,0) = mod_name;
+
+      ___release_scmobj (mod_name);
+
+      ___FIELD(descr,1) = *module->lp+___LS*___WS;
+
+      ___FIELD(___FIELD(module_descr,0),module_count) = descr;
+
+      ___release_scmobj (descr);
+
       module_count++;
     }
+
   return module->init_proc ();
 }
 
@@ -1494,7 +1520,15 @@ double x;)
 
 #else
 
-  return ((___FETCH_U16(&x,F64_HI16) ^ 0x7ff0) & 0x7fff) >= 0x10;
+  union
+    {
+      ___U16 u16[4];
+      ___F64 f64;
+    } y;
+
+  y.f64 = x;
+
+  return ((y.u16[F64_HI16] ^ 0x7ff0) & 0x7fff) >= 0x10;
 
 #endif
 }
@@ -1511,9 +1545,19 @@ double x;)
 
 #else
 
-  ___UM32 tmp = (___FETCH_U32(&x,F64_HI32) ^ 0x7ff00000) & 0x7fffffff;
+  ___UM32 tmp;
 
-  return tmp < 0x100000 && (tmp | ___FETCH_U32(&x,F64_LO32)) != 0;
+  union
+    {
+      ___U32 u32[2];
+      ___F64 f64;
+    } y;
+
+  y.f64 = x;
+
+  tmp = (y.u32[F64_HI32] ^ 0x7ff00000) & 0x7fffffff;
+
+  return tmp < 0x100000 && (tmp | y.u32[F64_LO32]) != 0;
 
 #endif
 }
@@ -1811,9 +1855,7 @@ ___SCMOBJ stack_marker;)
 
   for (;;)
     {
-#define CALL_STEP \
-___pc = ___CAST_FAKEHOST_TO_HOST(___CAST(___label_struct*,___pc-___tSUBTYPED) \
-                                 ->host)(___ps)
+#define CALL_STEP ___pc = ___LABEL_HOST(___pc)(___ps)
       CALL_STEP;
       CALL_STEP;
       CALL_STEP;
@@ -1986,6 +2028,9 @@ ___setup_params_struct *setup_params;)
   setup_params->terminal_settings = 0;
   setup_params->stdio_settings    = 0;
   setup_params->gambcdir          = 0;
+  setup_params->gambcdir_map      = 0;
+  setup_params->remote_dbg_addr   = 0;
+  setup_params->rpc_server_addr   = 0;
   setup_params->linker            = 0;
   setup_params->dummy8            = 0;
   setup_params->dummy7            = 0;
@@ -2265,6 +2310,12 @@ ___setup_params_struct *setup_params;)
   ___gstate.___U64_fits_in_width_fn
     = ___U64_fits_in_width_fn;
 
+  ___gstate.___U64_mul_UM32_UM32_fn
+    = ___U64_mul_UM32_UM32_fn;
+
+  ___gstate.___U64_add_U64_U64_fn
+    = ___U64_add_U64_U64_fn;
+
   ___gstate.___SCMOBJ_to_S8
     = ___SCMOBJ_to_S8;
 
@@ -2786,6 +2837,18 @@ ___setup_params_struct *setup_params;)
   ___gstate.___free_mem
     = ___free_mem;
 
+  ___gstate.___alloc_mem_code
+    = ___alloc_mem_code;
+
+  ___gstate.___free_mem_code
+    = ___free_mem_code;
+
+  ___gstate.___disable_heartbeat_interrupts
+    = ___disable_heartbeat_interrupts;
+
+  ___gstate.___enable_heartbeat_interrupts
+    = ___enable_heartbeat_interrupts;
+
 #endif
 
   /* 
@@ -2955,7 +3018,7 @@ ___setup_params_struct *setup_params;)
 
   if ((___err = ___make_sfun_stack_marker
                   (&marker,
-                   ___FIELD(___FIELD(___GSTATE->program_descr,0),0)))
+                   ___FIELD(___FIELD(___FIELD(___GSTATE->program_descr,0),0),1)))
       == ___FIX(___NO_ERR))
     {
       ___err = ___call (0, ___FIELD(marker,0), marker);
